/*
 * $Id: color.c,v 1.12 2007/05/20 13:19:12 simakov Exp $
 *
 * EPSILON - wavelet image compression library.
 * Copyright (C) 2006-2007 Alexander Simakov, <xander@entropyware.info>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * http://epsilon-project.sourceforge.net
 */

#include <common.h>
#include <color.h>

void convert_RGB_to_YCbCr(coeff_t **R, coeff_t **G, coeff_t **B,
                          coeff_t **Y, coeff_t **Cb, coeff_t **Cr,
                          int width, int height)
{
    int i, j;

    assert(width > 0);
    assert(height > 0);

    /* Perhaps one should optimize this code with integer-valued
     * look-up tables. On the other hand, the rest of the program
     * (e.g. wavelet transform, resampling etc.) still using FPU.
     * Moreover, additional rounding operation (even with proper
     * rescaling) will involve some marginal image distortion. */
    for (i = 0; i < height; i++) {
        for (j = 0; j < width; j++) {
            Y[i][j]  =  0.299000 * R[i][j] + 0.587000 * G[i][j] + 0.114000 * B[i][j];
            Cb[i][j] = -0.168736 * R[i][j] - 0.331264 * G[i][j] + 0.500000 * B[i][j] + 128.0;
            Cr[i][j] =  0.500000 * R[i][j] - 0.418688 * G[i][j] - 0.081312 * B[i][j] + 128.0;
        }
    }
}

void convert_YCbCr_to_RGB(coeff_t **Y, coeff_t **Cb, coeff_t **Cr,
                          coeff_t **R, coeff_t **G, coeff_t **B,
                          int width, int height)
{
    int i, j;

    assert(width > 0);
    assert(height > 0);

    /* Due to rounding and quantization errors output RGB values
     * may not be enclosed within [0..255] interval. To solve
     * this problem the values are clipped after transformation. */
    for (i = 0; i < height; i++) {
        for (j = 0; j < width; j++) {
            R[i][j] = CLIP(Y[i][j] + (Cr[i][j] - 128.0) * 1.402);
            G[i][j] = CLIP(Y[i][j] - (Cb[i][j] - 128.0) * 0.34413 - (Cr[i][j] - 128.0) * 0.71414);
            B[i][j] = CLIP(Y[i][j] + (Cb[i][j] - 128.0) * 1.772);
        }
    }
}

void convert_RGB_to_grayscale(coeff_t **R, coeff_t **G, coeff_t **B,
                              coeff_t **gray, int width, int height)
{
    int i, j;

    assert(width > 0);
    assert(height > 0);

    for (i = 0; i < height; i++) {
        for (j = 0; j < width; j++) {
            gray[i][j] = 0.299 * R[i][j] + 0.587 * G[i][j] + 0.114 * B[i][j];
        }
    }
}

void clip_channel(coeff_t **channel, int width, int height)
{
    int i, j;

    assert(width > 0);
    assert(height > 0);

    for (i = 0; i < height; i++) {
        for (j = 0; j < width; j++) {
            channel[i][j] = CLIP(channel[i][j]);
        }
    }
}
